from typing import Optional

from pydantic import BaseModel, Field, computed_field
from enum import Enum

from model.wapi.nd import BCD, Bldg, Module, Room, Port, Site


class PatchActionType(Enum):
    PATCH = 'patch'
    UNPATCH = 'unpatch'
    CHANGE_INSERT = 'change_insert'


class APIInsertBCDAssignment(BaseModel):
    bcd_name: str = Field()
    bcd_tagged: bool = Field(default=False)
    port_name: str = Field()


class APIPatchRequestAction(BaseModel):
    action: PatchActionType = Field()
    bcd_name: Optional[str] = Field(default=None)
    bcd_tagged: Optional[bool] = Field(default=False)
    building_gpk: str = Field()
    comment: str = Field()
    module_gpk: str = Field()
    room_gpk: str = Field()
    p_port_gpk: Optional[str] = Field(default=None)
    new_insert_type: Optional[str] = Field(default=None)
    new_insert_assignment: list[APIInsertBCDAssignment] = Field(default=[])


class PatchRequestAction(BaseModel):
    action: Optional[PatchActionType] = Field(default=None)
    bcd: Optional[BCD] = Field(default=None)
    bcd_tagged: Optional[bool] = Field(default=False)
    building: Optional[Bldg] = Field(default=None)
    comment: str = Field(default='')
    module: Optional[Module] = Field(default=None)
    room: Optional[Room] = Field(default=None)
    p_port: Optional[Port] = Field(default=None)
    site: Optional[Site] = Field(default=None)
    path_link: Optional[str] = Field(default=None)
    ports_summary: Optional[str] = Field(default=None)
    new_insert_type: Optional[str] = Field(default=None)
    new_insert_assignment: list[APIInsertBCDAssignment] = Field(default=[])
    vlan_ids: list[str] = Field(default=[])
    raw_path: Optional[list[dict]] = Field(default=None)


class PatchRequestFormRequest(BaseModel):
    actions: list[APIPatchRequestAction] = Field(default=[])
    reply_to: Optional[str] = Field(default=None)


class InformalPatchRequestFormRequest(BaseModel):
    site: Site = Field(default=None)
    comment: str = Field()
    reply_to: Optional[str] = Field(default=None)


class InsertPort(BaseModel):
    type: str = Field()
    type_group: str = Field()
    count: int = Field(default=1)
    proto: str = Field()
    name_prefix: str = Field(default='')
    name_suffix: str = Field(default='')

    @computed_field
    def name(self) -> str:
        return f'{self.name_prefix}%{{module_name}}{self.name_suffix}'

    @computed_field
    def can_assign_bcd(self) -> bool:
        return self.proto not in ['ISDN']


class InsertType(BaseModel):
    name: str = Field()
    ports: list[InsertPort] = Field(default=[])
